function [data]=bsac(varargin)
%BSAC   Arranges a timeseries into SAC structure format
%    Takes arrays of x-values and y-values
%    and format the arrays in a way that is compatible with
%    the SACLAB routines such as rsac, wsac, lh, gh, or ch. 
%
%    Usage: [data]=bsac(x_array1,y_array1,x_array2,y_array2,...)
%
%    Examples:
%
%    To create a square root function in matlab in the arrays
%    xarray and yarray, and then convert the array information
%    into SAC compatible format and ultimately write to a
%    SAC formatted binary file:
%
%    xarray=linspace(0,30,1000);
%    yarray=sqrt(xarray);
%    data=bsac(xarray,yarray);
%    data.name='myfile';
%    wsac(data);
%
%    note: only set up for time series data (and general
%          x vs y if you change the iftype header field
%          to 'ixy' afterwards)
%
%    by Michael Thorne (5/2004)   mthorne@asu.edu
%       Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also:  wsac, lh, ch, gh, rsac 

% check number of inputs
if (mod(nargin,2))
    error('Unpaired x/y arrays!')
end

% fill numerical header fields with default
h(1:110)=-12345;

% default character arrays
c8=[45 49 50 51 52 53 32 32];
c16=[45 49 50 51 52 53 32 32 32 32 32 32 32 32 32 32];

% fill character header fields with default
h(111:302)=[c8 c16 repmat(c8,1,21)];

% allocate data
data(nargin/2)=struct('head',[],'name',[]);

% loop for each pair
for i=1:2:nargin
    % output index
    j=round((i+1)/2);
    
    % check vector lengths
    [rows,columns]=size(varargin{i});
    if(((rows~=1)&&(columns~=1)) || (rows*columns<2))
        error('xarray is not a vector')
    end
    [rows,columns]=size(varargin{i+1});
    if(((rows~=1)&&(columns~=1)) || (rows*columns<2))
        error('xarray is not a vector')
    end
    
    % ensure column vectors
    varargin{i}=varargin{i}(:);
    varargin{i+1}=varargin{i+1}(:);
    
    % make sure lengths are the same
    npts=length(varargin{i});
    if (npts~=size(varargin{i+1}))
        error('series are not the same length')
    end
    
    % fill in amplitudes and times
    data(j).amp=varargin{i+1};
    data(j).head=h;
    
    % fill in knowns/presets
    data(j)=ch(data(j),...
        'delta',(varargin{i}(end)-varargin{i}(1))/(npts-1),...
        'b',varargin{i}(1),'e',varargin{i}(end),...
        'npts',npts,'depmin',min(varargin{i+1}),...
        'depmax',max(varargin{i+1}),'depmen',mean(varargin{i+1}),...
        'iftype',1,'leven',1,'lcalda',1,'lovrok',1,'lpspol',0,...
        'nvhdr',6,'kcmpnm','UNKNOWN','kstnm','SACLAB');
    
    % if timeseries is unevenly spaced add proper info
    if(abs((varargin{i}(end)-varargin{i}(1))/(npts-1)-(varargin{i}(2)-varargin{i}(1)))>eps)
        data(j).time=varargin{i};
        data(j)=ch(data(j),'leven',0,'odelta',varargin{i}(2)-varargin{i}(1));
    end
    
end

end
