function [data]=cut(data,ref1,offset1,ref2,offset2,fill)
%CUT    Cut SAC data
%
%       Cuts SAC data using the data window defined by
%       reference times and offsets.  Windows that 
%       exceed the bounds of the data are pushed to
%       the data bounds unless 'fill' is set to 1. In
%       this case, for evenly spaced files, zeros are
%       padded to fill the window.  For unevenly spaced
%       data the fill option is not available.
%
%   Usage: [data]=cut(data,ref1,offset1,ref2,offset2,fill)
%
%       ref values are one of the following:
%           'e','b','o','z','a','f','tn','n'
%
%           'e','b','o','a','f','tn' correspond to the
%           header values. 'z' is zero time. 'n' indicates
%           the offset is the sample number (starts with 1)
%           for the first reference.  For the second reference
%           the offset associated with 'n' is the cut window
%           length in number of samples.
%
%           If no ref value is given and an offset is, the
%           ref is assumed as 'z' (zero).  Offsets default
%           to 0 when not supplied.  If neither ref or
%           offset is given, then the defaults are
%           'b',0 or 'e',0.  If both pairs are missing, no
%           cut is done.
%
%       offset values are in seconds for all reference
%       values except 'n', in which case they are the 
%       sample number.
%
%       fill is only valid if set to 1.  Otherwise it is
%       ignored.  When set the window is padded with zeros
%       if the window exceeds the bounds of the data of 
%       evenly spaced files.
%
%   Examples:
%
%     cut files starting from the 33rd sample for 400 samples
%     [data]=cut(data,'n',33,'n',400);
%
%     cut out the t0 phase with some padding
%     [data]=cut(data,'t1',-30,'t1',60);
%
%     cut one hour after origin filling gaps with zeros
%     [data]=cut(data,'o',0,'o',3600,1);
%
%     cut first 300 seconds of files
%     [data]=cut(data,[],[],'b',300);
%      or explicitly
%     [data]=cut(data,'b',0,'b',300);
%
%     cut 300 to 500 seconds from reference time
%     [data]=cut(data,[],300,[],500);
%      or explicitly
%     [data]=cut(data,'z',300,'z',500);
%   
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: sacinterp, sacsample, deci, stretch

% input check
error(nargchk(1,6,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
end

% defaults
if(nargin==1)
    return;
end
if(nargin<6)
    fill=0;
end
if(nargin<5)
    offset2=0;
end
if(nargin<4)
    ref2='e';
end
if(nargin<3)
    offset1=0;
end
if(isempty(ref1))
    if(isempty(offset1))
        ref1='b';
    else
        ref1='z';
    end
end
if(isempty(ref2))
    if(isempty(offset2))
        ref2='e';
    else
        ref2='z';
    end
end
if(isempty(offset1))
    offset1=0;
end
if(isempty(offset2))
    offset2=0;
end

% checks
if(~ischar(ref1) || ~ischar(ref2))
    error('references must be strings')
elseif(~isnumeric(offset1) || ~isnumeric(offset2))
    error('offsets must be numeric')
end

% grab spacing info
[b,npts,delta,leven]=gh(data,'b','npts','delta','leven');

% number of seismograms
nseis=length(data);

% window begin time
if(strcmpi(ref1,'z'))
    bt(1:nseis,1)=offset1;
elseif(strcmpi(ref1,'n'))
    bp=fix(offset1);
else
    bt=gh(data,ref1)+offset1;
end

% window end time
if(strcmpi(ref2,'z'))
    et(1:nseis,1)=offset2;
elseif(strcmpi(ref2,'n'))
    ep=fix(offset2);
else
    et=gh(data,ref2)+offset2;
end

% loop through each file
for i=1:length(data)
    % check spacing for zero padding
    if(leven(i))
        % even case
        
        % calculate begin and end points
        if(~strcmpi(ref1,'n'))
            bp=round((bt(i)-b(i))/delta(i))+1;
        end
        if(strcmpi(ref2,'n'))
            ep2=bp+ep-1;
        else
            ep2=round((et(i)-b(i))/delta(i))+1;
        end
        
        % boundary conditions
        nbp=max([bp 1]);
        nep=min([ep2 npts(i)]);
        
        % cut
        data(i).amp=data(i).amp(nbp:nep);
        
        % fill or no
        if(fill)
            % add zeros
            data(i).amp=[zeros(1-bp,1); data(i).amp; zeros(ep2-npts(i),1)];
            
            % fix header
            data(i)=ch(data(i),'b',b(i)+(bp-1)*delta(i),...
                'e',b(i)+(ep2-1)*delta(i),'npts',max([0 ep2-bp+1]),...
                'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
                'depmax',max(data(i).amp));
        else
            % fix header
            data(i)=ch(data(i),'b',b(i)+(nbp-1)*delta(i),...
                'e',b(i)+(nep-1)*delta(i),'npts',max([0 nep-nbp+1]),...
                'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
                'depmax',max(data(i).amp));
        end
    else
        % uneven case
        
        % get begin point
        if(~strcmpi(ref1,'n'))
            bp=find(data(i).time>=bt(i),1);
        end
        
        % get end point
        if(strcmpi(ref2,'n'))
            ep2=bp+ep-1;
        else
            ep2=find(data(i).time<=et(i),1,'last');
        end

        % boundary conditions
        nbp=max([bp 1]);
        nep=min([ep2 npts(i)]);
        
        % cut
        data(i).amp=data(i).amp(nbp:nep);
        data(i).time=data(i).time(nbp:nep);
        
        % fix header
        data(i)=ch(data(i),'b',data(i).time(1),...
                'e',data(i).time(end),'npts',length(data(i).amp),...
                'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
                'depmax',max(data(i).amp));
    end
end
        

end
    