function [varargout]=gh(data,varargin)
%GH    get SAC header
%    Gets SAC header variables for SAC data
%    Values are returned in cell string arrays 
%    or numeric arrays oriented as a column vectors, 
%    with one value per cell/entry.  Fields are
%    case insensitive.
%
%    So asking for the 'delta' value will return
%    a numeric array with the first entry containing
%    the 'delta' value from the first sacfile in
%    the data structure, the second entry has the
%    second file's 'delta' value and so on.
%    
%    Usage:    [array_of_values1,...]=gh(sac_array,field1,...)
%
%    Examples:
%
%    [dt,depmin,depmax,...]=gh(data)  % gets all header variables
%    [dt]=gh(data,'DeLtA')
%    [stla,stlo]=gh(data,'stla','STLO')
%
%    by Garrett Euler (10/2007)  ggeuler@wustl.edu
%
%    See also:  ch, lh, rsac, bsac, wsac 

% do nothing on no input
if(nargin<1)
    return
end

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head'))
    error('sac data structure does not have proper fields')
end

% header names and positions
[numheader,enumheader,lgcheader,chrheader]=sachp();

% check for special lists
if(nargin==1)
    varargin=[fieldnames(numheader); fieldnames(enumheader);
        fieldnames(lgcheader); fieldnames(chrheader)];
elseif(nargin==2)
    if(strcmpi(varargin{1},'t'))
        varargin={'t0' 't1' 't2' 't3' 't4' 't5' 't6' 't7' 't8' 't9'};
    elseif(strcmpi(varargin{1},'kt'))
        varargin={'kt0' 'kt1' 'kt2' 'kt3' 'kt4' 'kt5' 'kt6' 'kt7' 'kt8' 'kt9'};
    elseif(strcmpi(varargin{1},'resp'))
        varargin={'resp0' 'resp1' 'resp2' 'resp3' 'resp4' 'resp5' 'resp6' 'resp7' 'resp8' 'resp9'};
    elseif(strcmpi(varargin{1},'user'))
        varargin={'user0' 'user1' 'user2' 'user3' 'user4' 'user5' 'user6' 'user7' 'user8' 'user9'};
    elseif(strcmpi(varargin{1},'kuser'))
        varargin={'kuser0' 'kuser1' 'kuser2'};
    end
end

% number of files
n=length(data);

% loop over fields
for i=1:length(varargin)
    % force lowercase
    field=lower(varargin{i});
    
    % preallocate output
    if(isfield(numheader,field) || isfield(enumheader,field) || isfield(lgcheader,field))
        varargin{i}=zeros(n,1);
    elseif(isfield(chrheader,field))
        varargin{i}=cell(n,1);
    else
        disp(sprintf('WARNING: %s IS NOT A KNOWN FIELD!',field))
    end
    
    % output based on type
    if(isfield(numheader,field))
        if(numheader.(field)<71)
            for j=1:n
                varargout{i}(j,1)=data(j).head(numheader.(field));
            end
        else
            for j=1:n
                varargout{i}(j,1)=round(data(j).head(numheader.(field)));
            end
        end
    elseif(isfield(enumheader,field))
        for j=1:n
            varargout{i}(j,1)=round(data(j).head(enumheader.(field)));
        end
    elseif(isfield(lgcheader,field))
        for j=1:n
            varargout{i}(j,1)=round(data(j).head(lgcheader.(field)));
        end
    elseif(isfield(chrheader,field))
        for j=1:n
            varargout{i}{j,1}=char(data(j).head(chrheader.(field)(1):chrheader.(field)(2)))';
        end
    end
end

end
