function p1(data,xlimits,ylimits,ncols) 
%P1   plotting utility for SAC files
%
%     plot SAC seismograms in individual subplots
%
%     Markers are drawn if times are set for the 'O', and
%     'T0-T9' header variables.  These markers are also
%     labeled if 'KT0-KT9' are also set.  
% 
%     Usage:  p1(data,xlim,ylim,ncols)
%
%       xlim and ylim are vectors with the form [time1 time2]
%       and [amp1 amp2] that indicate the range limits for the
%       plot.  These are optional and will scale to fit the
%       data by default.
%
%       ncols indicates the number of columns of subplots. The
%       default is 1 column.
%
%     Examples:
%
%
%     by Michael Thorne (5/2004)   mthorne@asu.edu
%        Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also:  p2, recsec

% return if no input
if(nargin<1)
    return
end

% check number of inputs
error(nargchk(1,4,nargin))

% initialize plot
figure;
whitebg('k')
set(gcf,'Name','P1 -- SAC Seismogram Plotting Utility', ...
    'NumberTitle','off','color','k', ...
    'Pointer','crosshair','PaperOrientation','landscape', ...
    'PaperPosition',[.5 .5 10 7.5],'PaperType','usletter');
set(gca,'FontName','times','FontWeight','light', ...
    'xcolor','w','ycolor','w', ...
	'TickDir','out','FontSize',4);

% check data structure
n=length(data);
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
end

% get header info
[b,npts,delta,leven,iftype]=gh(data,'b','npts','delta','leven','iftype');
[t0,t1,t2,t3,t4,t5,t6,t7,t8,t9]=gh(data,'t');
[kt0,kt1,kt2,kt3,kt4,kt5,kt6,kt7,kt8,kt9]=gh(data,'kt');
[o,ko,gcarc]=gh(data,'o','ko','gcarc');

% check if columns defined
if(nargin<4)
    ncols=1;
    nrows=n;
else
    nrows=ceil(n/ncols);
end

% loop through each file
for i=1:n
    % check if right filetype (if not skip)
    if(iftype(i)~=1 && iftype(i)~=4)
        continue;
    end
    
    % check if even or uneven timing
    if(leven(i))
        time=b(i)+(0:npts(i)-1)'*delta(i);
    else
        time=data(i).time;
    end
    
    % plot series
    subplot(nrows,ncols,i);
    plot(time,data(i).amp,'color','w')
    grid on
    
    % label last trace
    if(i==n)
        xlabel('Time (sec)')
    end
    
    % focus xaxis
    if(nargin>1 && ~isempty(xlimits))
        xlim(xlimits)
    end
    
    % focus yaxis
    if(nargin>2 && ~isempty(ylimits))
        ylim(ylimits)
    end
    
    % plot limits
    ylimits=get(gca,'ylim');
    xlimits=get(gca,'xlim');
    yrange=ylimits(2)-ylimits(1);
    xrange=xlimits(2)-xlimits(1);
    ypad=0.05*yrange;
    xpad=0.02*xrange;
    
    % plot origin
    hold on
    if(o(i)~=-12345)
        plot([o(i) o(i)]',[ylimits(1)+ypad ylimits(2)-2*ypad]',...
            'color','y','linewidth',3)
    end
    if(~strcmpi(ko{i}(1:6),'-12345'))
        text(o(i)+xpad,ylimits(2)-2*ypad,ko{i},'color','y', ...
            'FontName','times',...
            'VerticalAlignment','top',...
            'clipping','on',...
            'FontSize',4);
    else
        text(o(i)+xpad,ylimits(2)-2*ypad,'O','color','y', ...
            'FontName','times',...
            'VerticalAlignment','top',...
            'clipping','on',...
            'FontSize',4);
    end
    
    % plot picks
    for j=0:9
        value=eval(['t' num2str(j) '(' num2str(i) ')']);
        label=eval(['kt' num2str(j) '{' num2str(i) '}']);
        if(value~=-12345)
            plot([value value]',[ylimits(1)+ypad ylimits(2)-2*ypad]',...
                'color','r','linewidth',2)
        end
        if(~strcmpi(label(1:6),'-12345'))
            text(value+xpad,ylimits(2)-2*ypad,label,'color','r', ...
                'FontName','times',...
                'VerticalAlignment','top',...
                'clipping','on',...
                'FontSize',4);
        end
    end
    
    % plot trace name and gcarc
    title([texlabel(data(i).name,'literal') '  ' num2str(gcarc(i)) '\circ'])
    hold off
end

end
