function p2(data,xlimits,ylimits,legend_ok) 
%P2   plotting utility for SAC files
%
%     plot SAC seismograms read into matlab with rsac.m on top
%     of each other
% 
%     usage:  p2(data,xlim,ylim,legend_ok)
%
%     data represents a sac seismogram structure created
%     with rsac or bsac
%
%     xlim and ylim are vectors with the form [time1 time2]
%     and [amp1 amp2] that indicate the range limits for the
%     plot.  These are optional and will scale to fit the
%     data by default.
%
%     'legend_ok' equal to 1 turns on plotting the legend of
%     filenames.  The legend box can be moved by
%     left-clicking on the box, and dragging it to the desired
%     location.  The legend labels can also be edited by double-
%     clicking on them.
%
%     Examples:
%
%     To plot the first 4 seismograms in data without a time window
%
%     p2(data(1:4)) 
%
%     To plot the 5th and 8th seismograms in data with time window
%
%     p2(data([5 8]),[0 300]) 
%
%     To plot all traces, place no constraint on the time window and
%      add a legend
%
%     p2(data,[],1)
%
%     by Michael Thorne (5/2004)   mthorne@asu.edu
%        Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also:  p1, recsec

% return if no input
if(nargin<1)
    return
end

% check number of inputs
error(nargchk(1,4,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
end

% get header info
[b,npts,delta,leven,iftype]=gh(data,'b','npts','delta','leven','iftype');

% initialize plot
figure;
colors=lines;
whitebg('k')
set(gcf,'Name','P2 -- SAC Seismogram Plotting Utility', ...
    'NumberTitle','off','color','k', ...
    'Pointer','crosshair','PaperOrientation','landscape', ...
    'PaperPosition',[.5 2 10 4.5],'PaperType','usletter');
set(gca,'FontName','times','FontWeight','light', ...
    'xcolor','w','ycolor','w', ...
	'TickDir','out','FontSize',8);
xlabel('Time (sec)')
grid on
hold on

% loop through each file
for i=1:length(data)
    % check if right filetype (if not skip)
    if(iftype(i)~=1 && iftype(i)~=4)
        continue;
    end
    
    % check if even or uneven timing
    if(leven(i))
        time=b(i)+(0:npts(i)-1)'*delta(i);
    else
        time=data(i).time;
    end
    
    % plot series
    plot(time,data(i).amp,'color',colors(1+mod(i-1,7),:))
end
hold off

% limit time
if(nargin>1 && ~isempty(xlimits))
    xlim(xlimits);
end

% limit amplitude
if(nargin>2 && ~isempty(ylimits))
    ylim(ylimits);
end

% legend
if(nargin==4 && legend_ok)
    h=legend(data.name,'location','best');
    set(h,'interpreter','none')
    set(h,'fontsize',4)
end

end
