function [data]=rsac(varargin)
%RSAC    Read SAC binary files.
%    Reads in SAC (seismic analysis code) binary
%    format files into a data structure. Accepts
%    character arrays of filenames (one filename 
%    per row) and cell arrays of filenames (one 
%    filename per cell).
%
%    data.amp - amplitudes
%    data.time - times (if uneven)
%    data.head - SAC header information
%    data.name - the filename
%
%    Other fields for various filetypes
%     data.ampl - spectral amplitudes
%     data.phase - spectral phase
%     data.real - spectral real
%     data.imag - spectral imaginary
%     data.y - dependent variable (gen. xy)
%     data.x - independent variable (gen. xy) (if uneven)
%     data.z - matrix data (gen. xyz) (l2r,b2t)
%
%    Default read byte order is little-endian, 
%     but auto-switches to big-endian when necessary.
%
%    Usage:  [data]=rsac(['sacfile1'; 'sacfile2'; ...],...
%                       {'sacfile3' 'sacfile4' ...},...
%                       'sacfile5','sacfile6',...)
%
%    Examples:
%
%    data=rsac('KATH.R');
%    [b,npts,delta]=gh(data,'b','npts','delta');
%    plot(b+(0:npts-1)*delta,data.amp)
%
%    [data]=rsac('SQRL.R','AAK.R');
%    [data]=rsac(cellarray1,chrarray1,chrarray2,cellarray2);
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: wsac, bsac, ch, lh, gh

% checking and organizing filenames
for i=1:nargin
    % check that input is a char or cell array
    if(~ischar(varargin{i}) && ~iscell(varargin{i}))
        error('char and cell arrays only')
    end
    
    % force char to be cell and cell array to a column vector of cells
    varargin{i}=cellstr(varargin{i});
    varargin{i}=varargin{i}(:)';
end
varargin=[varargin{:}]';
n=length(varargin);

% allocating header array and data structure
h(1:302,1)=0;
data(n,1)=struct('name',[],'head',[]);

% loop for each file
for i=1:n
    % endian test
    try
        % try with little-endian first
        fid=fopen(varargin{i},'r','ieee-le');
    
        % reading header (single precision reals and some ints)
        h(1:70)=fread(fid,70,'single');
        h(71:77)=fread(fid,7,'int32');
    
        % checking version
        if (h(77)==4 || h(77)==5)
            fclose(fid);
            error(['NVHDR = 4 or 5. File: "' varargin{i} '" may be from an old version of SAC.'])
        elseif (h(77)~=6)
            fclose(fid);
            error(['Current rsac byte order: "little-endian". File: "' varargin{i} '" may be of opposite byte-order!'])
        end
    catch
        % now try big-endian
        fid=fopen(varargin{i},'r','ieee-be');
    
        % reading header (single precision reals and some ints)
        h(1:70)=fread(fid,70,'single');
        h(71:77)=fread(fid,7,'int32');
    
        % checking version
        if (h(77)==4 || h(77)==5)
            fclose(fid);
            error(['NVHDR = 4 or 5. File: "' varargin{i} '" may be from an old version of SAC.'])
        elseif (h(77)~=6)
            fclose(fid);
            error(['Current rsac byte order: "big-endian". File: "' varargin{i} '" may be of opposite byte-order!'])
        end
    end
    
    % reading header (rest of single precision ints and logicals)
    h(78:110)=fread(fid,33,'int32');
    
    % reading header (character strings)
    h(111:302)=fread(fid,192,'char');
    
    % output
    data(i).head=h;
    data(i).name=varargin{i};
    
    % act by file type
    if(h(86)==1)
        % time series
        data(i).amp=fread(fid,h(80),'single');
        
        % uneven independent variable stream
        if(~h(106))
            data(i).time=fread(fid,h(80),'single');
        end
    elseif(h(86)==2)
        % spectral file - real and imaginary
        data(i).real=fread(fid,h(80),'single');
        data(i).imag=fread(fid,h(80),'single');
    elseif(h(86)==3)
        % spectral file - amplitude and phase
        data(i).ampl=fread(fid,h(80),'single');
        data(i).phase=fread(fid,h(80),'single');
    elseif(h(86)==4)
        % general x vs y data (y is 'dependent')
        data(i).y=fread(fid,h(80),'single');
        
        % uneven independent variable stream
        if(~h(106))
            data(i).x=fread(fid,h(80),'single');
        end
    elseif(h(86)==51)
        % general xyz (3D) - x,y are evenly spaced
        data(i).z=fread(fid,h(80),'single');
    else
        % unknown filetype
        error('Unknown filetype');
    end
    
    % closing file
    fclose(fid);
end
