function [data]=sacinterp(data,dt,new_b,new_e,method)
%SACINTERP    Interpolate SAC data
%    Interpolates SAC data using 'interp1' 
%    to a new sample rate and updates header 
%    values.
%
%    Usage: [data]=sacinterp(data,dt,new_b,new_e,method)
%
%           dt is the new sample interval desired
%           new_b is the begin time for interpolation
%           new_e is the end time for interpolation
%           method is the interpolation type:
%              'pchip','linear','spline','nearest'
%
%    NOTE:  new_b, new_e, and method are 
%           optional and default to the 'b' and 'e' 
%           header values and the 'pchip' interpolation
%           method.  new_e won't be honored (ends up between
%           new_e-dt and new_e).
%          
%
%    Examples:
%
%     interpolate at 5 sps using b and e header values
%    data=sacinterp(data,0.2);  
%
%     interpolate at 1 sps from 300 seconds to e header value
%    data=sacinterp(data,1,300)
%
%     interpolate to 5 sps from 900 to 950 seconds using linear interp
%    data_pdiff=sacinterp(data,0.2,900,950,'linear')
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: sacsample, deci, stretch, sacfilter, xsacinterp

% check number of arguments
error(nargchk(2,5,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
end

% get begin, end, npts, and overwrite flag of data
[b,e,npts,delta,leven]=gh(data,'b','e','npts','delta','leven');

% read in overrides
if(nargin<5)
    method='pchip';
end
if(nargin<4 || isempty(new_e))
    new_e=e;
end
if(nargin<3 || isempty(new_b))
    new_b=b;
end

% number of seismograms, expand scalars, & checks
nseis=length(data);
if(isscalar(new_b))
    new_b(1:nseis,1)=new_b;
elseif(~isvector(new_b) || length(new_b)~=nseis)
    error('dimensions of new_b are bad')
end
if(isscalar(new_e))
    new_e(1:nseis,1)=new_e;
elseif(~isvector(new_e) || length(new_e)~=nseis)
    error('dimensions of new_e are bad') 
end
if(~isnumeric(dt))
    error('dt must be numeric')
elseif(isscalar(dt))
    dt(1:nseis,1)=dt;
elseif(~isvector(dt) || length(dt)~=nseis)
    error('dimensions of dt are bad') 
end
if(ischar(method) || iscellstr(method))
    method=cellstr(method);
else
    error('method must be a string')
end
if(isscalar(method))
    method(1:nseis,1)=method;
elseif(~isvector(method) || length(method)~=nseis)
    error('dimensions of method are bad')
end

% looping for each file
for i=1:nseis
    % make new timing array
    x=(new_b(i):dt(i):new_e(i))';
    
    % old timing of data
    if(leven(i))
        timing=b(i)+(0:npts(i)-1)'*delta(i);
    else
        timing=data(i).time;
    end
    
    % interpolate
    % NOTE: I choose Piecewise Cubic Hermite Interpolating Polynomial (PCHIP)
    %       because SPLINE will not preserve causal signals whereas PCHIP does
    %       to a much greater degree. Look at the matlab help for the 'pchip' 
    %       command to see more details.
    y=interp1(timing,data(i).amp,x,method{i});
    
    % update header
    data(i)=ch(data(i),'delta',dt(i),'b',x(1),'e',x(end),...
        'npts',length(y),'depmin',min(y),'leven',1,...
        'depmax',max(y),'depmen',mean(y));
    
    % write out
    data(i).amp=y;
end

end