function [data]=sacsample(data,new_delta)
%SACSAMPLE    Resamples SAC data
%     Resamples SAC data using matlab's resample
%     routine if possible, otherwise the problem is
%     passed on to sacinterp.  Note that resample
%     uses a cascade of interpolations (stretch) 
%     and decimatations to get to the new rate. 
%     Actually it doesn't but it can be thought of
%     in that way for simplicity.  The process avoids 
%     aliasing by using lowpass filtering before 
%     decimation.  When sacinterp is needed, aliasing
%     is avoided by pre-decimating the signal to a
%     lower samplerate then interpolating. There are
%     no options for cutting the series, use sacinterp
%     explicitly or use saccut.
%
%    Usage:  [data]=sacsample(data,delta)
%               delta: new sample interval desired
%
%    Examples:
%               change data to 5sps
%            [data]=sacsample(data,0.2)
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also: sacinterp, cut, sacfilter, deci, stretch

% check nargin
error(nargchk(2,2,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head') || ~isfield(data,'amp'))
    error('sac data structure does not have proper fields')
elseif(isfield(data,'time'))
    error('function does not work with unevenly sampled data')
end

% check delta
if(~isnumeric(new_delta))
    error('sample interval must be numeric')
elseif(new_delta<0)
    error('negative sample interval not allowed')
end

% read in header info
[old_delta]=gh(data,'delta');

% number of seismograms
nseis=length(data);

% make delta groups
[sort_old_delta,indices]=sort(old_delta);
j=1; b(j)=1; e(j)=1; g_del(j)=sort_old_delta(1);
for i=2:nseis
    if(sort_old_delta(i)~=sort_old_delta(b(j)))
        e(j)=i-1;
        j=j+1;
        b(j)=i;
        g_del(j)=sort_old_delta(i);
    end
end
e(j)=nseis;

% work on each group
ng=length(b);
for i=1:ng
    % find fraction expressed as integers
    [up,down]=rat(new_delta/g_del(i));
    
    % get prime factors of integers
    [decimators]=factor(down);
    [stretchors]=factor(up);

    % check cascade series for high primes
    if(~isempty(find(decimators>100,1)) || ...
        ~isempty(find(stretchors>100,1)) || up*down>2^31)
        disp('Interpolating because of large factors')
        
        % anti-aliasing step
        if(new_delta/g_del(i)<1)
            multi=ceil(g_del(i)/new_delta);
            [data(indices(b(i):e(i)))]=...
                sacresample(data(indices(b(i):e(i))),1,multi,new_delta);
        end
        
        % interpolating
        [data(indices(b(i):e(i)))]=...
            sacinterp(data(indices(b(i):e(i))),new_delta);
    else
        % resample amplitudes
        [data(indices(b(i):e(i)))]=...
            sacresample(data(indices(b(i):e(i))),up,down,new_delta);
    end
end

end

function [data]=sacresample(data,up,down,new_delta)
%SACRESAMPLE  Resample SAC array
%    Resamples a SAC array using an integer
%    ratio.  Sets appropriate header values.
%

% grab header info
[b,npts]=gh(data,'b','npts');

% build amp matrix
maxnpts=max(npts);
nseis=length(data);
amp=zeros(maxnpts,nseis);
for i=1:nseis
    amp(1:npts(i),i)=data(i).amp;
end

% resample amplitudes
[amp]=resample(amp,down,up);

% get new length
new_npts=floor((npts-1)*down/up)+1;

% assign data and update header
for i=1:nseis
    data(i).amp=amp(1:new_npts(i),i);
    data(i)=ch(data(i),'npts',new_npts(i),'depmen',mean(data(i).amp),...
        'depmin',min(data(i).amp),'depmax',max(data(i).amp),...
        'e',b(i)+(new_npts(i)-1)*new_delta,'delta',new_delta);
end

end
