function []=wsac(data)
%WSAC    Write SAC binary files.
%    Writes SAC (seismic analysis code) binary format files.
%    Set filenames in the data structure 'name' field.
%    To write files to a specific directory either
%    add the path to the filename or use the cd command.
%
%    Default byte order is little-endian, but can be set
%    to default to big-endian byte order.
%
%    Usage:   wsac(sac_data_structure)
%
%    Examples:
%
%    wsac(data)
%
%    by Garrett Euler (10/2007)   ggeuler@wustl.edu
%
%    See also:  rsac, bsac, lh, ch, gh 

% check number of inputs
error(nargchk(1,1,nargin))

% check data structure
if(~isstruct(data))
    error('sac data is not a structure')
elseif(~isfield(data,'name') || ~isfield(data,'head'))
    error('sac data structure does not have proper fields')
end

% loop over file/array pairs
for i=1:length(data)
    % open file for writing
    fid=fopen(data(i).name,'w','ieee-le');
    %fid=fopen(data(i).name,'w','ieee-be');
    
    % write single precision real header variables
    fwrite(fid,data(i).head(1:70),'single');
    
    % write single precision integer and logical header variables
    fwrite(fid,data(i).head(71:110),'int32');
    
    % write character header variables
    fwrite(fid,data(i).head(111:302),'char');
    
    % act by file type
    if(data(i).head(86)==1)
        % write amplitudes in time series
        fwrite(fid,data(i).amp,'single');
        
        % write uneven timing
        if(~data(i).head(106))
            fwrite(fid,data(i).time,'single');
        end
    elseif(data(i).head(86)==2)
        % write spectral file - real and imaginary
        fwrite(fid,data(i).real,'single');
        fwrite(fid,data(i).imag,'single');
    elseif(data(i).head(86)==3)
        % write spectral file - amplitude and phase
        fwrite(fid,data(i).ampl,'single');
        fwrite(fid,data(i).phase,'single');
    elseif(data(i).head(86)==4)
        % write dependent variable
        fwrite(fid,data(i).y,'single');
        
        % write uneven independent variable
        if(~data(i).head(106))
            fwrite(fid,data(i).x,'single');
        end
    elseif(data(i).head(86)==51)
        % write general xyz (3D)
        fwrite(fid,data(i).z,'single');
    end
    
    % close file
    fclose(fid);
end

end
